require 'spec_helper'

module SavingsPlansTest
  aws_billing_id = (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]
  describe 'SavingsPlans createSpSavedRecommendations: E2E tests' do
    include_context 'request metadata'

    # common method to test saved quote verification for each different
    # createSpSavedRecommendations request type in tests below
    def verify_quote_is_saved(created_quote)
      create_response_quote_id = created_quote[:node][:id]

      variables = {
        'ids': [
          create_response_quote_id
        ]
      }
      get_api_response(variables, headers, :spSavedRecommendations)
      expect_success_graphql
      expect(json_body[:data][:spSavedRecommendations][:edges]).not_to be_nil
      expect(json_body[:data][:spSavedRecommendations][:edges]).not_to be_empty
      sp_saved_recommendation = json_body[:data][:spSavedRecommendations][:edges][0]
      expect(sp_saved_recommendation[:node][:id]).not_to be_nil
      expect(sp_saved_recommendation[:node][:id]).to eq(create_response_quote_id)

      expect(sp_saved_recommendation[:node][:awsAccountId]).to eq(created_quote[:node][:awsAccountId])
      expect(sp_saved_recommendation[:node][:evaluationPeriod]).to eq(created_quote[:node][:evaluationPeriod])
      expect(sp_saved_recommendation[:node][:offeringType]).to eq(created_quote[:node][:offeringType])
      expect(sp_saved_recommendation[:node][:savingsPlanTerm]).to eq(created_quote[:node][:savingsPlanTerm])
      expect(sp_saved_recommendation[:node][:purchaseDate]).to eq(created_quote[:node][:purchaseDate])
    end

    # delete quote record, and verify spSavedRecommendations and
    # spSavedSimulation no longer return data for deleted quote
    def delete_quote_record_and_verify_deletion(quote_id)
      delete_variables = {
        recommendationId: quote_id
      }
      get_api_response(delete_variables, headers, :deleteSpSavedRecommendation)
      expect_success_graphql
      check(:deleteSpSavedRecommendation, service: service)

      # verify deleted quote no longer available via spSavedRecommendations
      saved_rec_variables = {
        'ids': [
          quote_id
        ]
      }
      get_api_response(saved_rec_variables, headers, :spSavedRecommendations)
      expect_success_graphql
      expect(json_body[:data][:spSavedRecommendations][:edges].length).to eq(0)

      # verify drilldown for deleted quote no longer available via spSavedSimulation
      wrap_request do
        saved_sim_variables = {
          recommendationId: quote_id
        }
        graphql url(service: service),
          format_request(
            :spSavedSimulation,
            service: service
          ),
          saved_sim_variables, headers
        expect(json_body[:data][:spSavedSimulation]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq('DataFetchingException')
      end

      # verify expected error if attempt made to delete previously deleted quote
      wrap_request do
        graphql url(service: service),
          format_request(
            :deleteSpSavedRecommendation,
            service: service
          ),
          delete_variables, headers
        expect(json_body[:errors][0][:extensions][:classification]).to eq('DataFetchingException')
        expect(json_body[:errors][0][:message]).to include('Savings Plan recommendations not found')
      end
    end

    # The below tests will create saved quotes in the db, and schedule jobs for
    # quote creation in the Allocation Scheduler in the env that they run in.
    it 'create, verify, and delete single quote for single account, HOURLY_COMMITMENT override, valid purchaseDate' do
      hourly_commit_override = 2
      future_date = "#{Date.today + 10}T00:00:00Z"
      variables = {
        input: [
          {
            awsAccountId: aws_billing_id,
            evaluationPeriod: 'THIRTY_DAYS',
            offeringType:  'PARTIAL_UPFRONT',
            savingsPlanTerm: 'THREE_YEAR',
            purchaseDate: future_date,
            overrideConfiguration: {
                overrideType: 'HOURLY_COMMITMENT',
                overrideHourlyCommitment: hourly_commit_override
            }
          }
        ]
      }
      wrap_request do
        graphql url(service: service),
          format_request(:createSpSavedRecommendations, service: service),
          variables, headers
        expect_success_graphql
        check(:createSpSavedRecommendations, service: service)
      end
      expect(json_body[:data][:createSpSavedRecommendations][:edges].length).to be 1

      created_quote = json_body[:data][:createSpSavedRecommendations][:edges][0]
      expect(created_quote[:node][:awsAccountId]).to eq(variables[:input][0][:awsAccountId])
      expect(created_quote[:node][:evaluationPeriod]).to eq(variables[:input][0][:evaluationPeriod])
      expect(created_quote[:node][:offeringType]).to eq(variables[:input][0][:offeringType])
      expect(created_quote[:node][:savingsPlanTerm]).to eq(variables[:input][0][:savingsPlanTerm])
      expect(created_quote[:node][:purchaseDate]).to eq(future_date)
      verify_quote_is_saved(created_quote)

      create_response_id = created_quote[:node][:id]
      # wait for quote creation job to complete
      quote_completion_check(create_response_id)

      # re-verify data consistency after quote creation completion
      verify_quote_is_saved(created_quote)

      # verify drilldown is available on successful quote creation
      sim_variables = {
          recommendationId: create_response_id
      }
      get_api_response(sim_variables, headers, :spSavedSimulation)
      # following line verifies that all drilldown fields have values consistent
      # with whats defined in formatter.yml for the spSavedSimulation route
      expect_success_graphql
      check(:spSavedSimulation, service: service)
      expect(json_body[:data][:spSavedSimulation][:hourlyCommitment].round(2)).to eq(hourly_commit_override.round(2))
      delete_quote_record_and_verify_deletion(create_response_id)
    end

    it 'create, verify, and delete single quote for single account, TOTAL_COVERAGE override, invalid purchaseDate' do
      coverage_override = 80
      invalid_future_date = "#{Date.today + 10}T00"
      variables = {
        input: [
          {
            awsAccountId: aws_billing_id,
            evaluationPeriod: 'THIRTY_DAYS',
            offeringType:  'PARTIAL_UPFRONT',
            savingsPlanTerm: 'THREE_YEAR',
            purchaseDate: invalid_future_date,
            overrideConfiguration: {
                overrideTotalCoverage: coverage_override,
                overrideType: 'TOTAL_COVERAGE'
            }
          }
        ]
      }
      # call sp saved rec route, record quote number
      get_api_response(variables, headers, :createSpSavedRecommendations)
      expect_success_graphql
      check(:createSpSavedRecommendations, service: service)
      expect(json_body[:data][:createSpSavedRecommendations][:edges].length).to be 1
      created_quote = json_body[:data][:createSpSavedRecommendations][:edges][0]
      expect(created_quote[:node][:awsAccountId]).to eq(variables[:input][0][:awsAccountId])
      expect(created_quote[:node][:evaluationPeriod]).to eq(variables[:input][0][:evaluationPeriod])
      expect(created_quote[:node][:offeringType]).to eq(variables[:input][0][:offeringType])
      expect(created_quote[:node][:savingsPlanTerm]).to eq(variables[:input][0][:savingsPlanTerm])
      # invalid purchaseDate format should be reformatted to 30 days in the future
      # Just checking for date, aliasing out time stamp to avoid potential flakiness
      expect(created_quote[:node][:purchaseDate]).to match(/#{Date.today + 30}T\d{2}:\d{2}:\d{2}Z/)
      verify_quote_is_saved(created_quote)

      # wait for quote creation job to complete
      create_response_id = created_quote[:node][:id]
      quote_completion_check(create_response_id)
      # re-verify data consistency after quote creation completion
      verify_quote_is_saved(created_quote)

      sim_variables = {
          recommendationId: create_response_id
      }
      get_api_response(sim_variables, headers, :spSavedSimulation)
      expect_success_graphql
      # following line verifies that all drilldown fields have values consistent
      # with whats defined in formatter.yml for the spSavedSimulation route
      check(:spSavedSimulation, service: service)
      # verify coverage override specified in create request, manifests in the drilldown
      expect(
          json_body[:data][:spSavedSimulation][:summary][:coverageAfter].round(2)).to eq((coverage_override.to_f / 100).round(2))

      delete_quote_record_and_verify_deletion(create_response_id)
    end

    it 'create, verify, and delete multiple quotes for multiple accounts, no override, no purchaseDate' do
      variables = {
        'input': [
          {
            awsAccountId: aws_billing_id,
            evaluationPeriod: 'SEVEN_DAYS',
            offeringType:  'ALL_UPFRONT',
            savingsPlanTerm: 'ONE_YEAR'
          },
          {
            awsAccountId: aws_billing_id,
            evaluationPeriod: 'THIRTY_DAYS',
            offeringType:  'PARTIAL_UPFRONT',
            savingsPlanTerm: 'ONE_YEAR'
          },
          {
            awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_3_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_3[:aws_billing_id],
            evaluationPeriod: 'SEVEN_DAYS',
            offeringType:  'NO_UPFRONT',
            savingsPlanTerm: 'THREE_YEAR'
          }
        ]
      }
      get_api_response(variables, headers, :createSpSavedRecommendations)
      expect_success_graphql
      check(:createSpSavedRecommendations, service: service)
      expect(json_body[:data][:createSpSavedRecommendations][:edges].length).to eq(3)
      created_quote_1 = json_body[:data][:createSpSavedRecommendations][:edges][0]
      created_quote_2 = json_body[:data][:createSpSavedRecommendations][:edges][1]
      created_quote_3 = json_body[:data][:createSpSavedRecommendations][:edges][2]
      # verify unique quote id's
      expect(created_quote_1[:node][:id]).not_to eq(created_quote_2[:node][:id])
      expect(created_quote_2[:node][:id]).not_to eq(created_quote_3[:node][:id])
      expect(created_quote_1[:node][:id]).not_to eq(created_quote_3[:node][:id])

      # we only want to test completion for quotes created for the consolidated redshirt account for the respective environment
      # The linked account may not have enough data for generating allocation results
      consolidated_acct_quotes = []
      json_body[:data][:createSpSavedRecommendations][:edges].each { |quote|
        if quote[:node][:awsAccountId] == aws_billing_id
          consolidated_acct_quotes << quote
        end
      }
      verify_quote_is_saved(created_quote_1)
      verify_quote_is_saved(created_quote_2)
      verify_quote_is_saved(created_quote_3)

      expect(consolidated_acct_quotes.length).to eq(2)
      create_response_id_quote_1 = consolidated_acct_quotes[0][:node][:id]
      create_response_id_quote_2 = consolidated_acct_quotes[1][:node][:id]
      # wait for quote creation job to complete
      quote_completion_check(create_response_id_quote_1)
      # re-verify data consistency after quote creation completion
      verify_quote_is_saved(consolidated_acct_quotes[0])
      quote_completion_check(create_response_id_quote_2)
      verify_quote_is_saved(consolidated_acct_quotes[1])

      # verify drilldown is available on successful quote creation
      # expect_success_graphql verifies that all drilldown fields have values consistent
      # with whats defined in formatter.yml for the spSavedSimulation route
      sim_variables_quote_1 = {
          recommendationId: create_response_id_quote_1
      }
      get_api_response(sim_variables_quote_1, headers, :spSavedSimulation)
      expect_success_graphql
      check(:spSavedSimulation, service: service)

      sim_variables_quote_2 = {
          recommendationId: create_response_id_quote_2
      }
      get_api_response(sim_variables_quote_2, headers, :spSavedSimulation)
      expect_success_graphql
      check(:spSavedSimulation, service: service)

      delete_quote_record_and_verify_deletion(create_response_id_quote_1)
      delete_quote_record_and_verify_deletion(create_response_id_quote_2)
    end

    # linked account used below will CURRENTLY produce NO_DATA quotes
    # tentatively marking below scenario as probable NO_DATA
    it 'create, verify, and delete single quote for linked account (probable NO_DATA)' do
      coverage_override = 80
      future_date = "#{Date.today + 10}T00:00:00Z"
      variables = {
        input: [
          {
            awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id],
            evaluationPeriod: 'THIRTY_DAYS',
            offeringType:  'PARTIAL_UPFRONT',
            savingsPlanTerm: 'THREE_YEAR',
            purchaseDate: future_date
          }
        ]
      }
      # call sp saved rec route, record quote number
      get_api_response(variables, headers, :createSpSavedRecommendations)
      expect_success_graphql
      check(:createSpSavedRecommendations, service: service)
      expect(json_body[:data][:createSpSavedRecommendations][:edges].length).to be 1
      created_quote = json_body[:data][:createSpSavedRecommendations][:edges][0]
      expect(created_quote[:node][:awsAccountId]).to eq(variables[:input][0][:awsAccountId])
      expect(created_quote[:node][:evaluationPeriod]).to eq(variables[:input][0][:evaluationPeriod])
      expect(created_quote[:node][:offeringType]).to eq(variables[:input][0][:offeringType])
      expect(created_quote[:node][:savingsPlanTerm]).to eq(variables[:input][0][:savingsPlanTerm])
      expect(created_quote[:node][:purchaseDate]).to eq(future_date)
      verify_quote_is_saved(created_quote)

      # wait for quote creation job to complete
      create_response_id = created_quote[:node][:id]
      quote_completion_check(create_response_id)
      # re-verify data consistency after quote creation completion
      verify_quote_is_saved(created_quote)

      # not verifying drilldown in this test, as linked redshirt account is currently
      # expected to produce NO_DATA recommendations. The call/output below
      # is just to help keep track of if that status changes
      wrap_request do
        sim_variables = {
          recommendationId: create_response_id
        }
        graphql url(service: service),
          format_request(
            :spSavedSimulation,
            service: service
          ),
          sim_variables, headers
        if json_body[:data][:spSavedSimulation] != nil
          puts "linked account used in test now has data associated with it"
        end
      end
      delete_quote_record_and_verify_deletion(create_response_id)
    end

    it 'when both commit and coverage override values are provided in input,
          only the value related to the specified overrideType is applied' do
      # initially create quote without any overrides
      initial_variables = {
        input: [
          {
            awsAccountId: aws_billing_id,
            evaluationPeriod: 'THIRTY_DAYS',
            offeringType:  'PARTIAL_UPFRONT',
            savingsPlanTerm: 'THREE_YEAR'
          }
        ]
      }
      get_api_response(initial_variables, headers, :createSpSavedRecommendations)
      expect_success_graphql
      check(:createSpSavedRecommendations, service: service)
      quote_id = json_body[:data][:createSpSavedRecommendations][:edges][0][:node][:id]
      quote_completion_check(quote_id)

      # get default commit and coverage values for quote type
      id_variables = {
        'ids': [
          quote_id
        ]
      }
      get_api_response(id_variables, headers, :spSavedRecommendations)
      initial_hourly_commit =
          json_body[:data][:spSavedRecommendations][:edges][0][:node][:recommendedHourlyCommitment]
      initial_coverage =
          json_body[:data][:spSavedRecommendations][:edges][0][:node][:savingsPlanRecommendationSummary][:coverageAfter] * 100
      expect_success_graphql
      check(:spSavedRecommendations, service: service)

      # generate override values based whats already available for quote type in redshirt
      # whether we select to increase or decrease coverage depends on existing coverage
      # this is to avoid test flakiness for test, i.e. if coverage is already 90, no point in
      # setting coverage override to 90. Also have to account for possible default low coverage
      # to emphasize disparity between commit and coverage overrides (for test verification purposes),
      # increase commit if decreasing coverage, and vice versa
      if initial_coverage > 50
        coverage_override = 40
        commit_override = initial_hourly_commit * 2
        reduced_coverage = true
      else
        coverage_override = 70
        commit_override = initial_hourly_commit / 2
      end

      # make create quote request with both new override values, but TOTAL_COVERAGE overrideType
      override_variables = {
        input: [
          {
            awsAccountId: aws_billing_id,
            evaluationPeriod: 'THIRTY_DAYS',
            offeringType:  'PARTIAL_UPFRONT',
            savingsPlanTerm: 'THREE_YEAR',
            overrideConfiguration: {
              overrideType: 'TOTAL_COVERAGE',
              overrideTotalCoverage: coverage_override,
              overrideHourlyCommitment: commit_override
            }
          }
        ]
      }
      get_api_response(override_variables, headers, :createSpSavedRecommendations)
      expect_success_graphql
      check(:createSpSavedRecommendations, service: service)
      new_quote_id = json_body[:data][:createSpSavedRecommendations][:edges][0][:node][:id]
      # a new quote creation job is currently not being kicked off at this stage, as the
      # quote was just created. The overrides are just applied to the newly created quote
      # including quote completion check here to future proof test against any changes to this behavior
      quote_completion_check(new_quote_id)

      new_id_variables = {
        'ids': [
          new_quote_id
        ]
      }
      get_api_response(new_id_variables, headers, :spSavedRecommendations)
      new_commit = json_body[:data][:spSavedRecommendations][:edges][0][:node][:recommendedHourlyCommitment]
      new_coverage =
          json_body[:data][:spSavedRecommendations][:edges][0][:node][:savingsPlanRecommendationSummary][:coverageAfter] * 100

      # verify coverage override gets applied, while commit override does not
      expect(new_coverage.round(2)).to eq(coverage_override.round(2))
      expect(new_commit.round(3)).not_to eq(commit_override.round(3))
      # expect commit to decrease if coverage override decreases coverage, and vice versa
      if reduced_coverage
        expect(new_commit.round(3)).to be < commit_override.round(3)
      else
        expect(new_commit.round(3)).to be > commit_override.round(3)
      end
      # delete quote ids created during test
      delete_quote_record_and_verify_deletion(quote_id)
      delete_quote_record_and_verify_deletion(new_quote_id)
    end

    # Separating these tests until new feature is made public
    # after which all tests will specify scope
    # Including all possible fields for regression test purposes
    # TODO: unskip pending resolution of https://cloudhealthtech.atlassian.net/browse/ENG-33752
    xcontext 'Temporary tests for SP COGS feature' do
      it 'create, verify, and delete single quote with BILLING_FAMILY scope (with coverage override)' do
        coverage_override = 80
        future_date = "#{Date.today + 10}T00:00:00Z"
        variables = {
          input: [
            {
              awsAccountId: aws_billing_id,
              evaluationPeriod: 'CUSTOMIZED_PERIOD',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              accountScope: 'BILLING_FAMILY',
              analysisStartDate: "#{Date.today - 10}T00:00:00Z",
              analysisEndDate: "#{Date.today}T00:00:00Z",
              purchaseDate: future_date,
              overrideConfiguration: {
                overrideType: 'TOTAL_COVERAGE',
                overrideTotalCoverage: coverage_override
              }
            }
          ]
        }
        get_api_response(variables, headers, :createSpSavedRecommendations)
        expect_success_graphql
        check(:createSpSavedRecommendations, service: service)
        expect(json_body[:data][:createSpSavedRecommendations][:edges].length).to be 1
        created_quote = json_body[:data][:createSpSavedRecommendations][:edges][0]
        expect(created_quote[:node][:awsAccountId]).to eq(variables[:input][0][:awsAccountId])
        expect(created_quote[:node][:evaluationPeriod]).to eq(variables[:input][0][:evaluationPeriod])
        expect(created_quote[:node][:offeringType]).to eq(variables[:input][0][:offeringType])
        expect(created_quote[:node][:savingsPlanTerm]).to eq(variables[:input][0][:savingsPlanTerm])
        expect(created_quote[:node][:purchaseDate]).to eq(future_date)

        create_response_id = created_quote[:node][:id]
        quote_completion_check(create_response_id)
        verify_quote_is_saved(created_quote)

        sim_variables = {
            recommendationId: create_response_id
        }
        get_api_response(sim_variables, headers, :spSavedSimulation)
        expect_success_graphql
        check(:spSavedSimulation, service: service)
        expect(
            json_body[:data][:spSavedSimulation][:summary][:coverageAfter].round(2)).to eq((coverage_override.to_f / 100).round(2))

        delete_quote_record_and_verify_deletion(create_response_id)
      end

      it 'create, verify, and delete single quote with SINGLE_ACCOUNT scope (with commit override)' do
        hourly_commit_override = 2
        future_date = "#{Date.today + 10}T00:00:00Z"
        variables = {
          input: [
            {
              awsAccountId: aws_billing_id,
              evaluationPeriod: 'CUSTOMIZED_PERIOD',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              accountScope: 'SINGLE_ACCOUNT',
              analysisStartDate: "#{Date.today - 10}T00:00:00Z",
              analysisEndDate: nil,
              purchaseDate: future_date,
              overrideConfiguration: {
                overrideType: 'HOURLY_COMMITMENT',
                overrideHourlyCommitment: hourly_commit_override
              }
            }
          ]
        }
        get_api_response(variables, headers, :createSpSavedRecommendations)
        expect_success_graphql
        check(:createSpSavedRecommendations, service: service)
        expect(json_body[:data][:createSpSavedRecommendations][:edges].length).to be 1
        created_quote = json_body[:data][:createSpSavedRecommendations][:edges][0]
        expect(created_quote[:node][:awsAccountId]).to eq(variables[:input][0][:awsAccountId])
        expect(created_quote[:node][:evaluationPeriod]).to eq(variables[:input][0][:evaluationPeriod])
        expect(created_quote[:node][:offeringType]).to eq(variables[:input][0][:offeringType])
        expect(created_quote[:node][:savingsPlanTerm]).to eq(variables[:input][0][:savingsPlanTerm])
        expect(created_quote[:node][:purchaseDate]).to eq(future_date)

        create_response_id = created_quote[:node][:id]
        quote_completion_check(create_response_id)
        verify_quote_is_saved(created_quote)

        sim_variables = {
            recommendationId: create_response_id
        }
        get_api_response(sim_variables, headers, :spSavedSimulation)
        expect_success_graphql
        check(:spSavedSimulation, service: service)
        expect(json_body[:data][:spSavedSimulation][:hourlyCommitment].round(2)).to eq(hourly_commit_override.round(2))
        delete_quote_record_and_verify_deletion(create_response_id)
      end

    end
  end

  describe 'SavingsPlans createSpSavedRecommendations: Negative tests' do
    include_context 'request metadata'

    # TODO: Add low target coverage test on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-28491
    it 'returns expected error for no auth request' do
      wrap_request do
        variables = {
          'input': [
            {
              awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id],
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR'
            }
          ]
        }
        graphql url(service: service),
                format_request(
                    :createSpSavedRecommendations,
                    service: service
                ),
                variables, apitest_headers
        expect_status(403)
      end
    end

    it 'route returns expected error when awsAccountId is invalid' do
      wrap_request do
        variables = {
          'input': [
            {
              awsAccountId: 'invalid_id',
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR'
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :createSpSavedRecommendations,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:errorType]).to eq("BAD_REQUEST")
      end
    end

    it 'route returns expected error when desired quote not included' do
      wrap_request do
        variables = {
          'input': [
            {
              awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id]
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :createSpSavedRecommendations,
            service: service
          ),
          variables, headers
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
      end
    end

    it 'returns expected error for overrideHourlyCommitment request without HOURLY_COMMITMENT overrideType specified' do
      wrap_request do
        variables = {
          input: [
            {
              awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id],
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              overrideConfiguration: {
                overrideHourlyCommitment: 1
              }
            }
          ]
        }
        graphql url(service: service),
                format_request(
                    :createSpSavedRecommendations,
                    service: service
                ),
                variables, headers
        expect(json_body[:data]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
      end
    end

    it 'returns expected error for overrideTotalCoverage request without TOTAL_COVERAGE overrideType specified' do
      wrap_request do
        variables = {
          input: [
            {
              awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id],
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              overrideConfiguration: {
                  overrideTotalCoverage: 90
              }
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :createSpSavedRecommendations,
            service: service
          ),
          variables, headers
        expect(json_body[:data]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
      end
    end

    it 'returns expected error when create request specifies wrong (non-redshirt) account id' do
      wrap_request do
        variables = {
          input: [
            {
              awsAccountId: INCORRECT_CUSTOMER[:aws_billing_id],
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR'
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :createSpSavedRecommendations,
            service: service
          ),
          variables, headers
        expect(json_body[:data][:createSpSavedRecommendations]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq('DataFetchingException')
        expect(
            json_body[:errors][0][:message]).to include(
            'AWS account is either not accessible, or does not exist within your organization, or does not have a payer account')
      end
    end

    it 'returns expected error for override type mismatch' do
      wrap_request do
        variables = {
          input: [
            {
              awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id],
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              overrideConfiguration: {
                overrideHourlyCommitment: 1,
                overrideType: 'TOTAL_COVERAGE'
              }
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :createSpSavedRecommendations,
            service: service
          ),
          variables, headers
        expect(json_body[:data][:createSpSavedRecommendations]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
      end
    end

    it 'returns expected error for HOURLY_COMMITMENT override with 0 commit' do
      wrap_request do
        variables = {
          input: [
            {
              awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id],
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              overrideConfiguration: {
                overrideType: 'HOURLY_COMMITMENT',
                overrideHourlyCommitment: 0
              }
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :createSpSavedRecommendations,
            service: service
          ),
          variables, headers
        expect(json_body[:data][:createSpSavedRecommendations]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
      end
    end

    it 'returns expected error for TOTAL_COVERAGE override with 0 coverage' do
      wrap_request do
        variables = {
          input: [
            {
              awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id],
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              overrideConfiguration: {
                overrideType: 'TOTAL_COVERAGE',
                overrideTotalCoverage: 0
              }
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :createSpSavedRecommendations,
            service: service
          ),
          variables, headers
        expect(json_body[:data][:createSpSavedRecommendations]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
      end
    end

    # expect a BILLING_FAMILY scope quote request will eventually
    # be rejected when made against a linked account
    # TODO: unskip pending resolution of https://cloudhealthtech.atlassian.net/browse/ENG-34058
    xit 'cannot make use BILLING_FAMILY scope for linked acct' do
      wrap_request do
        variables = {
          input: [
            {
              awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id],
              evaluationPeriod: 'CUSTOMIZED_PERIOD',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              accountScope: 'BILLING_FAMILY',
              analysisStartDate: "#{Date.today}T00:00:00Z",
              analysisEndDate: "#{Date.today}T00:00:00Z"
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :createSpSavedRecommendations,
            service: service
          ),
          variables, headers
        expect_status(400)
        expect(json_body[:data][:createSpSavedRecommendations]).to be_nil
      end
    end

    # TODO: unskip on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-34105
    xit 'analysisStartDate cannot be nil when evaluationPeriod is CUSTOMIZED_PERIOD' do
      wrap_request do
        variables = {
          input: [
            {
              awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id],
              evaluationPeriod: 'CUSTOMIZED_PERIOD',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              accountScope: 'BILLING_FAMILY',
              analysisStartDate: nil,
              analysisEndDate: nil
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :createSpSavedRecommendations,
            service: service
          ),
          variables, headers
        expect(json_body[:data][:createSpSavedRecommendations]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq('DataFetchingException')
      end
    end

    # TODO: unskip on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-34105
    xit 'analysisStartDate must be in ISO 8601 format' do
      wrap_request do
        variables = {
          input: [
            {
              awsAccountId: aws_billing_id,
              evaluationPeriod: 'CUSTOMIZED_PERIOD',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              accountScope: 'BILLING_FAMILY',
              analysisStartDate: "September 29th 2021",
              analysisEndDate: nil
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :createSpSavedRecommendations,
            service: service
          ),
          variables, headers
        expect(json_body[:data][:createSpSavedRecommendations]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq('DataFetchingException')
      end
    end

    it 'analysisEndDate must be in ISO 8601 format' do
      wrap_request do
        variables = {
          input: [
            {
              awsAccountId: aws_billing_id,
              evaluationPeriod: 'CUSTOMIZED_PERIOD',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              accountScope: 'BILLING_FAMILY',
              analysisStartDate: "#{Date.today - 10}T00:00:00Z",
              analysisEndDate: "2021/19/10",
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :createSpSavedRecommendations,
            service: service
          ),
          variables, headers
        expect(json_body[:data][:createSpSavedRecommendations]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq('ValidationError')
        expect(json_body[:errors][0][:message]).to include('Dates must be in ISO format')
      end
    end

    # TODO: unskip all following tests pending resolution of https://cloudhealthtech.atlassian.net/browse/ENG-33813
    xit 'analysisEndDate cannot be future date' do
      wrap_request do
        variables = {
          input: [
            {
              awsAccountId: aws_billing_id,
              evaluationPeriod: 'CUSTOMIZED_PERIOD',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              accountScope: 'BILLING_FAMILY',
              analysisStartDate: "#{Date.today - 10}T00:00:00Z",
              analysisEndDate: "#{Date.today}T00:00:00Z"
            }
          ]
        }
        graphql url(service: service),
          format_request(
              :createSpSavedRecommendations,
              service: service
          ),
          variables, headers
        expect(json_body[:data][:createSpSavedRecommendations]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq('DataFetchingException')
      end
    end

    xit 'analysisStartDate cannot be future date' do
      wrap_request do
        variables = {
          input: [
            {
              awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id],
              evaluationPeriod: 'CUSTOMIZED_PERIOD',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              accountScope: 'BILLING_FAMILY',
              analysisStartDate: "#{Date.today + 10}T00:00:00Z",
              analysisEndDate: nil
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :createSpSavedRecommendations,
            service: service
          ),
          variables, headers
        expect(json_body[:data][:createSpSavedRecommendations]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq('DataFetchingException')
      end
    end

    xit 'analysisEndDate cannot be earlier than analysisStartDate' do
      wrap_request do
        variables = {
          input: [
            {
              awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id],
              evaluationPeriod: 'CUSTOMIZED_PERIOD',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              accountScope: 'BILLING_FAMILY',
              analysisStartDate: "#{Date.today}T00:00:00Z",
              analysisEndDate: "#{Date.today - 10}T00:00:00Z"
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :createSpSavedRecommendations,
            service: service
          ),
          variables, headers
        expect(json_body[:data][:createSpSavedRecommendations]).to be_nil
        expect(json_body[:errors][0][:extensions][:classification]).to eq('DataFetchingException')
      end
    end
  end

  describe 'SavingsPlans createSpSavedRecommendations: Negative tests (SavingsPlans x API Proxy)' do
    include_context 'request metadata'

    before(:example) do
      @override_endpoint = ENV.delete('OVERRIDE_ENDPOINT')
    end

    after(:example) do
      ENV['OVERRIDE_ENDPOINT'] = @override_endpoint
    end

    # temp test to verify new fields not available until api-proxy exposure
    it 'requests with non-public fields gets rejected by api-proxy' do
      wrap_request do
        future_date = "#{Date.today + 10}T00:00:00Z"
        variables = {
          input: [
            {
              awsAccountId: aws_billing_id,
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
              accountScope: 'BILLING_FAMILY',
              analysisStartDate: "#{Date.today}T00:00:00Z",
              analysisEndDate: future_date,
              purchaseDate: future_date,
              overrideConfiguration: {
                overrideType: 'TOTAL_COVERAGE',
                overrideTotalCoverage: 0
              }
            }
          ]
        }
        graphql url(service: ApiProxySavingsPlans.new(via_public: via_public)),
          format_request(
            :createSpSavedRecommendations,
            service: ApiProxySavingsPlans.new
          ),
          variables, headers
        expect_status(400)
      end
    end
  end
end
