require 'rspec/json_expectations'

module SimulationNodeTests
  RSpec.shared_examples 'standard simulation tests' do |simulation_options, recommendation_type, expected_overcommit, request_body |
    include_context 'request metadata'
    simulation_node = {}
    summary_node = {}
    cost_data_node = {}
    usage_data_node = {}
    ec2_operating_systems = ['rhel', 'windows', 'linux']
    paginated_response = false

    before(:each) {
      if simulation_node.empty?
        if recommendation_type == :spSavedSimulation
          rec_id = ''
          complete_quote_found = false
          variables = {
              'filters': request_body
          }
          # get quote types based in variables passed in from test
          get_api_response(variables, headers, :spSavedRecommendations)
          # grab first quote found that is COMPLETED
          json_body[:data][:spSavedRecommendations][:edges].each { |quote|
            if quote[:node][:status] == 'COMPLETED'
              rec_id = quote[:node][:id]
              complete_quote_found = true
            end
            break if complete_quote_found
          }

          # a saved quote needs to get to COMPLETED stage before simulation is available
          if complete_quote_found == false
            create_variables = {
                input: request_body
            }
            get_api_response(create_variables, headers, :createSpSavedRecommendations)
            rec_id = json_body[:data][:createSpSavedRecommendations][:edges][0][:node][:id]
            quote_completion_check(rec_id)
          end
          simulation_options[:recommendationId] = rec_id
        end

        # use default commit (maximum savings) from associated recommendation
        # when hourlyCommitment set to 'default_commit' in drilldown options
        if recommendation_type == :savingsPlanSimulation &&
            simulation_options[:options][:hourlyCommitment] == 'default_commit'
          recommendation_request_options = {
            options: {
              awsBillingAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
              evaluationPeriod: simulation_options[:options][:evaluationPeriod],
              considerExpiringUsage: true
            }
          }
          get_api_response(recommendation_request_options, headers, :savingsPlanRecommendations)
          expect_success_graphql
          recommendations_array = json_body[:data][:savingsPlanRecommendations][:recommendations][:edges]
          recommendations_array.each { |rec|
            if rec[:node][:savingsPlanConfiguration][:term] == simulation_options[:options][:term] &&
                rec[:node][:savingsPlanConfiguration][:offeringType] == simulation_options[:options][:offeringType]
              simulation_options[:options][:hourlyCommitment] = rec[:node][:recommendedHourlyCommitment]
            end
          }
        end

        if expected_overcommit == true
          # get hourly commit for 100 coverage use that + x for override commit
          # cover both spSavedSimulation and savingsPlanSimulation cases
          if recommendation_type == :spSavedSimulation
            new_simulation_options = {
              recommendationId: rec_id,
              override: {
                'overrideTotalCoverage': 100,
                'overrideType': 'TOTAL_COVERAGE'
              }
            }
            get_api_response(new_simulation_options, headers, recommendation_type)
            new_hourly_commit = json_body[:data][recommendation_type][:hourlyCommitment] + 100
            simulation_options[:override][:overrideHourlyCommitment] = new_hourly_commit
          else
            total_coverage_hourly_commit = 0
            recommendation_request_options = {
              options: {
                awsBillingAccountId: simulation_options[:options][:awsBillingAccountId],
                evaluationPeriod: simulation_options[:options][:evaluationPeriod],
                overrideConfiguration: {
                  overrideTotalCoverage: 100,
                  overrideType: 'TOTAL_COVERAGE'
                }
              }
            }
            get_api_response(recommendation_request_options, headers, :savingsPlanRecommendations)
            # get the hourly commit for the quote type that matches the savingsPlanSimulation case under test
            json_body[:data][:savingsPlanRecommendations][:recommendations][:edges].each { |rec|
              if (rec[:node][:savingsPlanConfiguration][:offeringType] == simulation_options[:options][:offeringType]) &&
                  (rec[:node][:savingsPlanConfiguration][:term] == simulation_options[:options][:term])
                total_coverage_hourly_commit =  rec[:node][:recommendedHourlyCommitment]
              end
            }
            new_hourly_commit = total_coverage_hourly_commit + 100
            simulation_options[:options][:hourlyCommitment] = new_hourly_commit
          end
        end

        if simulation_options[:first] != nil || simulation_options[:last] != nil
          graphql url(service: service),
            format_request(
              recommendation_type,
              service: service, paginate: ['usageData']
            ),
            simulation_options, headers
          expect_status(200)
          expect_success_graphql
          paginated_response = true
        else
          get_api_response(simulation_options, headers, recommendation_type)
        end

        simulation_node = json_body[:data][recommendation_type]
        if simulation_node.nil?
          raise Exception, 'api request returned nil simulation node'
        elsif simulation_node.empty?
          raise Exception, 'api request returned empty simulation node'
        end

        summary_node = simulation_node[:summary]
        cost_data_node = simulation_node[:costData]
        usage_data_node = simulation_node[:usageData]

      end
    }

    it 'route returns all expected objects as per schema' do
      check(recommendation_type, service: service)
    end

    it 'an over-commit should lead to unusedCommit with values > 0' do
      if expected_overcommit
        unusedCommitTotal = simulation_node[:costData][:unusedCommit].inject(0) {|sum, i|  sum + i }
        expect(unusedCommitTotal).to be > 0.0
      else
        skip('overcommit not expected')
      end
    end

    it '100% coverage should be returned for over-commit' do
      if expected_overcommit
        expect(simulation_node[:summary][:coverageAfter].round(2)).to eq(1.0)
      else
        skip('overcommit not expected')
      end
    end

    it 'hourlyCommitment in response is the same as hourlyCommitment requested' do
      if recommendation_type == :spSavedSimulation &&
          (simulation_options[:override] == nil || simulation_options[:override][:overrideHourlyCommitment] == nil)
        skip('no hourlyCommitment in request')
      else
        if recommendation_type == :spSavedSimulation
          hourlyCommit = simulation_options[:override][:overrideHourlyCommitment].round(2)
        else
          hourlyCommit = simulation_options[:options][:hourlyCommitment].round(2)
        end
        expect(simulation_node[:hourlyCommitment].round(2)).to eq (hourlyCommit)
      end
    end

    it 'totalcoverage in response is the same as totalcoverage override' do
      if recommendation_type == :savingsPlanSimulation ||
          (simulation_options[:override] == nil || simulation_options[:override][:overrideTotalCoverage] == nil)
        skip('no coverage override in request')
      else
        expectedCoverage = simulation_options[:override][:overrideTotalCoverage].to_f / 100
        expect(summary_node[:coverageAfter].round(2)).to eq (expectedCoverage.round(2))
      end
    end

    it 'totalUpfrontCost is correct for each recommendation, based on recommendedHourlyCommitment' do
      num_of_years = get_sp_term_as_int(simulation_node[:savingsPlanConfiguration][:term])
      offering_type = simulation_node[:savingsPlanConfiguration][:offeringType]
      annual_cost = simulation_node[:hourlyCommitment] * 8760

      if offering_type == 'ALL_UPFRONT'
        expected_total_cost = num_of_years * annual_cost
      elsif  offering_type == 'PARTIAL_UPFRONT'
        # using ceil value for partial upfront, due to https://github.com/CloudHealth/cht-svc-savings-plans/pull/91
        expected_total_cost = ((annual_cost / 2) * num_of_years).ceil
      else
        expected_total_cost = 0
      end

      expect(expected_total_cost).not_to be_nil
      expect(summary_node[:totalUpfrontCost]).not_to be_nil
      expect(summary_node[:totalUpfrontCost]).to be >= 0
      expect(summary_node[:totalUpfrontCost].round()).to eq(expected_total_cost.round())
    end

    it 'monthlySavings corresponds with lifetimeSavings for each recommendation' do
      num_of_years = get_sp_term_as_int(simulation_node[:savingsPlanConfiguration][:term])
      if summary_node[:monthlySavings] > 0
        expected_monthly_savings = summary_node[:lifetimeSavings] * 729.6 / (num_of_years * 8760)
        expect(expected_monthly_savings).not_to be_nil
        tolerance_factor = summary_node[:monthlySavings] * 0.01
        expect(summary_node[:monthlySavings]).to be_within(tolerance_factor).of(expected_monthly_savings)
      else
        puts "Negative monthlySavings for request: #{simulation_options}"
      end
    end

    it 'coverageAfter in each Recommendation Summary > coverageBefore' do
      expect(summary_node[:coverageBefore]).not_to be_nil
      expect(summary_node[:coverageBefore]).to be >= 0.round
      expect(summary_node[:coverageAfter]).not_to be_nil
      if summary_node[:coverageBefore] < 0.8
        expect(summary_node[:coverageAfter]).to be > summary_node[:coverageBefore],
          "coverageBefore should be < coverageAfter for: #{summary_node}"
      end
    end

    it 'globalDiscountRateAfter in each Recommendation Summary > globalDiscountRateBefore' do
      if summary_node[:monthlySavings] > 0
        expect(summary_node[:globalDiscountRateBefore]).to be >= 0
        expect(summary_node[:globalDiscountRateAfter])
          .to be > summary_node[:globalDiscountRateBefore]
      end

    end

    it 'existingOnDemandCost and onDemandCostCovered arrays are the same size' do
      expect(cost_data_node[:existingOnDemandCost]).not_to be_nil
      expect(cost_data_node[:existingOnDemandCost].length).to be > 0
      expect(cost_data_node[:onDemandCostCovered]).not_to be_nil
      expect(cost_data_node[:onDemandCostCovered].length).to be > 0

      expect(cost_data_node[:existingOnDemandCost].length)
        .to eq(cost_data_node[:onDemandCostCovered].length)
    end

    it 'existingOnDemandCost total is greater than onDemandCostCovered total' do
      on_demand_cost_covered_sum =
        cost_data_node[:onDemandCostCovered].inject(0) {|sum, i|  sum + i }
      existing_on_demand_cost_covered_sum =
        cost_data_node[:existingOnDemandCost].inject(0) {|sum, i|  sum + i }
      expect(existing_on_demand_cost_covered_sum).to be > 0
      # if the input hourly commitment results in 100% coverage, existingOnDemandCost = onDemandCostCovered
      if (summary_node[:coverageAfter] >
          summary_node[:coverageBefore]) && summary_node[:coverageAfter] < 0.99
        expect(existing_on_demand_cost_covered_sum).to be > on_demand_cost_covered_sum
      end

    end

    context 'For SavingsPlan usageData (table) node/array:' do
      it 'usageData array is not null or empty' do
        expect(usage_data_node).not_to be_nil
        expect(usage_data_node[:edges]).not_to be_nil
        expect(usage_data_node[:edges].length).to be > 0
      end

      it 'each usageData node should have a valid monthly savings value' do
        non_zero_monthly_savings = false
        total_monthly_savings = 0
        usage_data_node[:edges].each{|element|
          expect(element[:node][:monthlySavings]).to be >= 0, "for: #{element}"
          total_monthly_savings += element[:node][:monthlySavings]
          non_zero_monthly_savings = true
        }
        if non_zero_monthly_savings == false
          raise Exception, "no monthlySavings node had a value > 0. Please check test acct usage: #{json_body}"
        end

        if expected_overcommit == false && paginated_response == false
          # The following test accounts for the following assumptions:
          # without unused commitment, sum of usageData monthlySavings values == monthly savings summary value
          # with unused commitment, sum of usageData monthlySavings values can be > monthly savings summary value
          # monthly savings in the summary is the overall monthly savings
          # monthly savings in each row, is the savings from each analysis point,
          # and does not include unused commitment, which can make savings higher
          expect(total_monthly_savings.round()).to be >= (summary_node[:monthlySavings].round())
        end
      end

      it 'each usageData node should have a valid totalCost value' do
        non_zero_total_cost = false
        usage_data_node[:edges].each{|element|
          expect(element[:node][:totalCost]).to be >= 0.0, "for: #{element}"
          non_zero_total_cost = true
        }
        if non_zero_total_cost == false
          raise Exception, "no totalCost node had a value > 0. Please check test acct usage: #{json_body}"
        end
      end

      it 'a Lambda and Fargate node should be present' do
        lambda_fargate_node_count = 0
        usage_data_node[:edges].each{|element|
          instance_family = element[:node][:instanceFamily].to_s.downcase
          if instance_family.include?('lambda') || instance_family.include?('fargate')
            expect(element[:node][:operatingSystem]).to be_nil, "for: #{element}"
            # expecting a region format such as us-east-2
            expect(element[:node][:region]).to match(/\w+\-\w+\-\d/)
            lambda_fargate_node_count += 1
          end
        }
        # only testing fargate/lambda node presence for 30 day period, to allow max
        # fault tolerance for testing on redshirt account. Remove below test if redshirt
        # usage becomes too unreliable to reliably test lambda/fargate node presence
        if simulation_node[:savingsPlanConfiguration][:term].to_s.upcase == 'THIRTY_DAYS'
          expect(lambda_fargate_node_count).to be > 0,
            "no lambda or farget nodes in usageData node #{usage_data_node}"
        end

      end

      it 'an EC2 node should be present with valid values' do
        ec2_node_count = 0
        usage_data_node[:edges].each{|element|
          if ec2_operating_systems.include? element[:node][:operatingSystem].to_s.downcase
            expect(element[:node][:instanceFamily]).not_to be_nil, "for: #{element}"
            instance_family = element[:node][:instanceFamily].to_s.downcase
            expect(instance_family).not_to be_empty
            expect(instance_family).not_to eq('lambda and fargate'), "for: #{element}"

            # expecting a region format such as us-east-2
            expect(element[:node][:region]).to match(/\w+\-\w+\-\d/)

            expect(element[:node][:tenancy]).not_to be_nil, "for: #{element}"
            # All other nil checks are covered in formatters.yml
            ec2_node_count += 1
          end
        }
        expect(ec2_node_count).to be > 0, "no ec2 nodes in usageData node #{usage_data_node}"
      end

      it 'savings plan coverage should correspond with monthly savings' do
        applicable_nodes = 0

        if recommendation_type == :spSavedSimulation
          offering_type = request_body[0][:offeringType]
          savings_plan_term = request_body[0][:savingsPlanTerm]
        else
          offering_type = simulation_options[:options][:offeringType]
          savings_plan_term = simulation_options[:options][:term]
        end
        usage_data_node[:edges].each{|element|
          expect(element[:node][:monthlySavings]).not_to be_nil
          expect(element[:node][:monthlySavings]).to be >= 0
          # lamda and mac1 (1 year no upfront) nodes not included at this time
          # as Amazon offers 0 % discount for such instances
          unless element[:node][:instanceFamily].to_s.downcase.include?('lambda') ||
              (element[:node][:instanceFamily].to_s.downcase.include?('mac1') &&
                  offering_type == 'NO_UPFRONT' &&
                  savings_plan_term == 'ONE_YEAR')
            # savingsPercent value is monthlySavings as a % of totalCost
            savingsPercent =
              (element[:node][:monthlySavings] * 
                  get_sp_term_as_int(simulation_node[:savingsPlanConfiguration][:term])) / 
                  element[:node][:totalCost]

            if element[:node][:monthlySavings] > 1 && savingsPercent > 0.05
              expect(element[:node][:savingsPlanCoverageBefore]).to be < element[:node][:savingsPlanCoverageAfter],
                "savingsPlanCoverageBefore should be < savingsPlanCoverageAfter for: #{element}"
            end
            applicable_nodes =+ 1
          end
        }
        # the below verification is just to verify the above loop is entered into
        # and the tests inside run on at least one usageData node
        expect(applicable_nodes).to be > 0,
          "coverage tests did not run on any node, please check current instanceFamily format
           to ensure above condition is correct. usageData node: #{JSON::pretty_generate(usage_data_node)}"
      end
    end
  end

  # The following tests verify behavior post implementation of ENG-34210
  # The default recommendation maximizes savings (minimizes spend)
  # If you start at the default recommendation and increase OR decrease hourly commitment,
  # the expectation is that monthly spend will go up.
  # In both cases below, estimated monthly spend should increase
  RSpec.shared_examples 'Override hourly commit beyond optimum savings in drilldown' do |eval_period|
    rec_id = ''
    initial_spend = 0
    initial_commit = 0
    create_rec_node = {}

    before(:each) {
      if create_rec_node.empty?
        request_body = {
          input: [
            {
              awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
              evaluationPeriod: eval_period,
              offeringType:  'ALL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR'
            }
          ]
        }
        get_api_response(request_body, headers, :createSpSavedRecommendations)
        create_rec_node = json_body[:data][:createSpSavedRecommendations][:edges][0][:node]
        rec_id = create_rec_node[:id]
        quote_completion_check(rec_id)
        variables = {
          recommendationId: rec_id
        }
        get_api_response(variables, headers, :spSavedSimulation)
        initial_spend = json_body[:data][:spSavedSimulation][:summary][:monthlySpend]
        initial_commit = json_body[:data][:spSavedSimulation][:hourlyCommitment]
      end
    }

    it "#{eval_period}: expect commit reduction to lead to increased monthly costs" do
      commit_override = initial_commit / 2
      variables = {
        recommendationId: rec_id,
        override: {
          overrideHourlyCommitment: commit_override,
          overrideType: 'HOURLY_COMMITMENT'
        }
      }
      get_api_response(variables, headers, :spSavedSimulation)
      spend_after = json_body[:data][:spSavedSimulation][:summary][:monthlySpend]
      expect(spend_after).to be > initial_spend
    end

    it "#{eval_period}: expect commit increase to lead to increased monthly costs" do
      commit_override = initial_commit + 1
      variables = {
        recommendationId: rec_id,
        override: {
          overrideHourlyCommitment: commit_override,
          overrideType: 'HOURLY_COMMITMENT'
        }
      }
      get_api_response(variables, headers, :spSavedSimulation)
      spend_after = json_body[:data][:spSavedSimulation][:summary][:monthlySpend]
      expect(spend_after).to be > initial_spend
    end
  end
end
