require 'spec_helper'

module SavingsPlansTest
  aws_billing_id = (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id]

  RSpec.shared_examples 'standard pagination tests' do |pagination_options, options, simulationType|
    include_context 'request metadata'

    it 'Pagination: first and after' do
      expected_first_cursor =
        pagination_options[:usage_node_array_unpaginated][pagination_options[:mid_cursor_index] + 1]
      expected_end_cursor =
        pagination_options[:usage_node_array_unpaginated][pagination_options[:mid_cursor_index] + pagination_options[:first_and_last_value]]

      simulation_options = {
        first: pagination_options[:first_and_last_value],
        after: pagination_options[:mid_cursor]
      }

      if simulationType == :spSavedSimulation
        simulation_options[:recommendationId] = options[:recommendationId]
      else
        simulation_options[:options] = options
      end

      graphql url(service: service),
        format_request(
          simulationType,
          service: service, paginate: ['usageData']
        ),
        simulation_options, headers

      check(simulationType, service: service)
      expect_status(200)
      expect_success_graphql
      page_info = json_body[:data][simulationType][:usageData][:pageInfo]
      usage_nodes = json_body[:data][simulationType][:usageData][:edges]
      expect(usage_nodes[0][:cursor]).to eq(expected_first_cursor)
      expect(page_info[:startCursor]).to eq(usage_nodes[0][:cursor])
      expect(page_info[:endCursor]).to eq(expected_end_cursor)
      expect(page_info[:hasNextPage]).to eq(true)
      expect(page_info[:hasPreviousPage]).to eq(true)
      expect(usage_nodes.length).to be > 0
      expect(usage_nodes.length).to eq(pagination_options[:first_and_last_value])
    end

    it 'Pagination: last and before' do
      expected_first_cursor =
        pagination_options[:usage_node_array_unpaginated][pagination_options[:mid_cursor_index] - pagination_options[:first_and_last_value]]
      expected_end_cursor =
        pagination_options[:usage_node_array_unpaginated][pagination_options[:mid_cursor_index] - 1]

      simulation_options = {
        last: pagination_options[:first_and_last_value],
        before: pagination_options[:mid_cursor]
      }

      if simulationType == :spSavedSimulation
        simulation_options[:recommendationId] = options[:recommendationId]
      else
        simulation_options[:options] = options
      end

      graphql url(service: service),
        format_request(
          simulationType,
          service: service, paginate: ['usageData']
        ),
        simulation_options, headers

      check(simulationType, service: service)
      expect_status(200)
      expect_success_graphql
      page_info = json_body[:data][simulationType][:usageData][:pageInfo]
      usage_nodes = json_body[:data][simulationType][:usageData][:edges]
      expect(usage_nodes[0][:cursor]).to eq(expected_first_cursor)
      expect(usage_nodes.length).to eq(pagination_options[:first_and_last_value])
      expect(page_info[:startCursor]).to eq(usage_nodes[0][:cursor])
      expect(page_info[:endCursor]).to eq(expected_end_cursor)
      expect(page_info[:hasNextPage]).to eq(true)
      expect(page_info[:hasPreviousPage]).to eq(true)
    end

    it 'Pagination: no next page' do
      second_last_cursor_index =  pagination_options[:usage_node_array_unpaginated].length - 2

      simulation_options = {
        first: 1,
        after: pagination_options[:usage_node_array_unpaginated][second_last_cursor_index]
      }

      if simulationType == :spSavedSimulation
        simulation_options[:recommendationId] = options[:recommendationId]
      else
        simulation_options[:options] = options
      end

      wrap_request do
        graphql url(service: service),
          format_request(
            simulationType,
            service: service, paginate: ['usageData']
          ),
          simulation_options, headers
        expect_success_graphql
        check(simulationType, service: service)
      end
      page_info = json_body[:data][simulationType][:usageData][:pageInfo]
      expect(page_info[:hasPreviousPage]).to eq(true)
      expect(page_info[:hasNextPage]).to eq(false)
    end

    it 'Unpaginated Total count and summary monthly savings values persist in paginated response' do
      expected_first_cursor =
        pagination_options[:usage_node_array_unpaginated][pagination_options[:mid_cursor_index] + 1]
      expected_end_cursor =
        pagination_options[:usage_node_array_unpaginated][pagination_options[:mid_cursor_index] + pagination_options[:first_and_last_value]]

      simulation_options = {
        first: pagination_options[:first_and_last_value],
        after: pagination_options[:mid_cursor]
      }

      if simulationType == :spSavedSimulation
        simulation_options[:recommendationId] = options[:recommendationId]
      else
        simulation_options[:options] = options
      end

      graphql url(service: service),
        format_request(
          simulationType,
          service: service, paginate: ['usageData']
        ),
        simulation_options, headers

      check(simulationType, service: service)
      expect_status(200)
      expect_success_graphql
      simulation_node = json_body[:data][simulationType]
      usage_data_node = json_body[:data][simulationType][:usageData]
      paginated_response_total_count = usage_data_node[:totalCount]
      paginated_response_monthly_savings = simulation_node[:summary][:monthlySavings]
      expect(paginated_response_total_count).to eq(pagination_options[:total_count_unpaginated])
      expect(paginated_response_monthly_savings).to eq(pagination_options[:monthly_savings_unpaginated])

      if usage_data_node[:edges].length < pagination_options[:usage_node_array_unpaginated].length
        total_paginated_monthly_savings = 0
        usage_data_node[:edges].each{|usage_node|
          total_paginated_monthly_savings += usage_node[:node][:monthlySavings]
        }
        expect(total_paginated_monthly_savings).to be < pagination_options[:monthly_savings_unpaginated]
      end
    end

  end

  # the below tests contain 2 test conexts for each simulation type
  # one context runs pagination specific tests for each simulation type (shared example above)
  # the other context runs the standard simulation tests on paginated requests
  describe 'savingsPlanSimulation pagination tests' do
    include_context 'request metadata'
    usage_cursor_array_unpaginated = []
    pagination_options = {}

    before(:each) {
      if usage_cursor_array_unpaginated.empty?
        simulation_options = {
          options: {
            awsBillingAccountId: aws_billing_id,
            considerExpiringUsage: false,
            evaluationPeriod: 'THIRTY_DAYS',
            hourlyCommitment: 3.0,
            offeringType: 'ALL_UPFRONT',
            term: 'ONE_YEAR',
            type: 'COMPUTE'
          }
        }
        wrap_request do
          graphql url(service: service),
            format_request(
              :savingsPlanSimulation,
              service: service
            ),
            simulation_options, headers
          check(:savingsPlanSimulation, service: service)
          expect_status(200)
          expect_success_graphql
        end
        simulation_node = json_body[:data][:savingsPlanSimulation]
        usage_data_node = json_body[:data][:savingsPlanSimulation][:usageData]
        usage_data_node[:edges].each { |recommendation|
          usage_cursor_array_unpaginated << recommendation[:cursor]
        }
        if usage_cursor_array_unpaginated.empty?
          raise Exception.new "There are no recommendations available for tests to use"
        end
        # the below is intended to ensure page range is always
        # within valid range regardless of test data being used
        mid_cursor_index =  usage_cursor_array_unpaginated.length/ 2
        pagination_options[:total_count_unpaginated] = usage_data_node[:totalCount]
        pagination_options[:monthly_savings_unpaginated] = simulation_node[:summary][:monthlySavings]
        pagination_options[:usage_node_array_unpaginated] = usage_cursor_array_unpaginated
        pagination_options[:first_and_last_value] = (usage_cursor_array_unpaginated.length / 2).round() - 2
        pagination_options[:mid_cursor_index] =  mid_cursor_index
        pagination_options[:mid_cursor] = usage_cursor_array_unpaginated[mid_cursor_index]
      end
    }

    context 'savingsPlanSimulation standard pagination tests' do
      options = {
        awsBillingAccountId: aws_billing_id,
        considerExpiringUsage: false,
        evaluationPeriod: 'THIRTY_DAYS',
        hourlyCommitment: 3.0,
        offeringType: 'ALL_UPFRONT',
        term: 'ONE_YEAR',
        type: 'COMPUTE'
      }
      include_examples 'standard pagination tests', pagination_options, options, :savingsPlanSimulation
    end

    context 'savingsPlanSimulation standard simulation tests with pagination' do
      simulation_options = {}
      before(:each) {
        options = {
          awsBillingAccountId: aws_billing_id,
          considerExpiringUsage: false,
          evaluationPeriod: 'THIRTY_DAYS',
          hourlyCommitment: 3.0,
          offeringType: 'ALL_UPFRONT',
          term: 'ONE_YEAR',
          type: 'COMPUTE'
        }
        simulation_options[:options] = options
        simulation_options[:first] = pagination_options[:first_and_last_value]
        simulation_options[:after] = pagination_options[:mid_cursor]
      }
      include_examples 'standard simulation tests', simulation_options, :savingsPlanSimulation
    end
  end


  describe 'spSavedSimulation pagination tests tests' do
    include_context 'request metadata'
    usage_cursor_array_unpaginated = []
    pagination_options = {}
    request_options = {}
    rec_id = ''

    before(:each) {
      if usage_cursor_array_unpaginated.empty?
        request_body = [
          {
            awsAccountId: aws_billing_id,
            evaluationPeriod: 'SEVEN_DAYS',
            offeringType:  'ALL_UPFRONT',
            savingsPlanTerm: 'ONE_YEAR'
          }
        ]
        complete_quote_found = false
        variables = {
            'filters': request_body
        }
        # get quote types based in variables passed in from test
        get_api_response(variables, headers, :spSavedRecommendations)
        # grab first quote found that is COMPLETED
        json_body[:data][:spSavedRecommendations][:edges].each { |quote|
          if quote[:node][:status] == 'COMPLETED'
            rec_id = quote[:node][:id]
            complete_quote_found = true
          end
          break if complete_quote_found
        }

        # a saved quote needs to get to COMPLETED stage before simulation is available
        if complete_quote_found == false
          create_variables = {
              input: request_body
          }
          get_api_response(create_variables, headers, :createSpSavedRecommendations)
          rec_id = json_body[:data][:createSpSavedRecommendations][:edges][0][:node][:id]
          quote_completion_check(rec_id)
        end
        simulation_options = {
            recommendationId: rec_id
        }
        get_api_response(simulation_options, headers, :spSavedSimulation)
        simulation_node = json_body[:data][:spSavedSimulation]
        usage_data_node = json_body[:data][:spSavedSimulation][:usageData]
        usage_data_node[:edges].each { |recommendation|
          usage_cursor_array_unpaginated << recommendation[:cursor]
        }
        if usage_cursor_array_unpaginated.empty?
          raise Exception.new "There are no recommendations available for tests to use"
        end
        # the below is intended to ensure page range is always
        # within valid range regardless of test data being used
        mid_cursor_index =  usage_cursor_array_unpaginated.length/ 2
        pagination_options[:total_count_unpaginated] = usage_data_node[:totalCount]
        pagination_options[:monthly_savings_unpaginated] = simulation_node[:summary][:monthlySavings]
        pagination_options[:usage_node_array_unpaginated] = usage_cursor_array_unpaginated
        pagination_options[:first_and_last_value] = (usage_cursor_array_unpaginated.length / 2).round() - 2
        pagination_options[:mid_cursor_index] =  mid_cursor_index
        pagination_options[:mid_cursor] = usage_cursor_array_unpaginated[mid_cursor_index]
        request_options[:recommendationId] = rec_id
      end
    }

    context 'spSavedSimulation standard pagination tests' do
      include_examples 'standard pagination tests', pagination_options, request_options, :spSavedSimulation
    end

    context 'spSavedSimulation standard simulation tests with pagination' do
      simulation_options = {}
      before(:each) {
        simulation_options[:first] = pagination_options[:first_and_last_value]
        simulation_options[:after] = pagination_options[:mid_cursor]
      }

      request_body = [
        {
          awsAccountId: aws_billing_id,
          evaluationPeriod: 'SEVEN_DAYS',
          offeringType:  'ALL_UPFRONT',
          savingsPlanTerm: 'ONE_YEAR'
        }
      ]
      include_examples 'standard simulation tests', simulation_options, :spSavedSimulation, false, request_body
    end
  end
end
