require 'spec_helper'

module SavingsPlansTest
  describe 'Positive tests: SavingsPlans spSavedRecommendations route' do
    include_context 'request metadata'
    request_body = [
      {
        awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
        evaluationPeriod: 'THIRTY_DAYS',
        offeringType:  'PARTIAL_UPFRONT',
        savingsPlanTerm: 'THREE_YEAR'
      },
      {
        awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
        evaluationPeriod: 'SEVEN_DAYS',
        offeringType:  'ALL_UPFRONT',
        savingsPlanTerm: 'THREE_YEAR'
      }
    ]

    before(:each) {
      create_saved_recs_if_none_exist(request_body)
    }

    it 'returns recommendations/quotes by filter' do
      variables = {
        'filters': request_body
      }
      get_api_response(variables, headers, :spSavedRecommendations)
      expect_success_graphql
      check(:spSavedRecommendations, service: service)

      # verify that the quotes we get back match the quotes we requested
      requestedQuotes = []
      variables[:filters].each { |rec|
        requestedQuotes <<
          "#{rec[:awsAccountId]}/#{rec[:evaluationPeriod]}/#{rec[:offeringType]}/#{rec[:savingsPlanTerm]}"
      }

      quotesReturned = []
      json_body[:data][:spSavedRecommendations][:edges].each { |rec|
        quotesReturned <<
          "#{rec[:node][:awsAccountId]}/#{rec[:node][:evaluationPeriod]}/#{rec[:node][:offeringType]}/#{rec[:node][:savingsPlanTerm]}"
      }
      # expecation is you can get more that one quote,
      # if there are numerous quotes of the same type associated with an account
      expect(quotesReturned.length).to be >= variables[:filters].length
      quotesReturned.each { |quote|
        expect(requestedQuotes).to include quote
      }
    end

    it 'returns recommendations/quotes by id' do
      filter_variables = {
        'filters': [
          {
            awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
            evaluationPeriod: 'THIRTY_DAYS',
            offeringType:  'PARTIAL_UPFRONT',
            savingsPlanTerm: 'THREE_YEAR',
          }
        ]
      }
      get_api_response(filter_variables, headers, :spSavedRecommendations)
      id = json_body[:data][:spSavedRecommendations][:edges][0][:node][:id]

      id_variables = {
        'ids': [
          id
        ]
      }
      get_api_response(id_variables, headers, :spSavedRecommendations)
      expect_success_graphql
      check(:spSavedRecommendations, service: service)

      recommendations_array = json_body[:data][:spSavedRecommendations][:edges]
      expect(recommendations_array.length).to eq(1)
      
      returned_quote = recommendations_array[0][:node]
      expect(returned_quote[:id]).to eq(id)
      expect(returned_quote[:awsAccountId]).to eq(filter_variables[:filters][0][:awsAccountId])
      expect(returned_quote[:evaluationPeriod]).to eq(filter_variables[:filters][0][:evaluationPeriod])
      expect(returned_quote[:offeringType]).to eq(filter_variables[:filters][0][:offeringType])
      expect(returned_quote[:savingsPlanTerm]).to eq(filter_variables[:filters][0][:savingsPlanTerm])
    end
    
    context 'recommendations summary tests' do
      include_context 'request metadata'
      filter_variables = {
        'filters': [
          {
            awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
            evaluationPeriod: 'THIRTY_DAYS',
            offeringType:  'PARTIAL_UPFRONT',
            savingsPlanTerm: 'THREE_YEAR'
          }
        ]
      }
      include_examples 'recommendation summary tests', filter_variables, :spSavedRecommendations
    end
  end

  describe 'Negative tests: SavingsPlans spSavedRecommendations route' do
    include_context 'request metadata'

    it 'returns expected error for no auth request' do
      wrap_request do
        variables = {
          'filters': [
            {
              awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_CUSTOMER_PREPROD[:aws_billing_id] : TEST_CUSTOMER[:aws_billing_id],
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :spSavedRecommendations,
            service: service
          ),
          variables, apitest_headers
        expect_status(403)
      end
    end

    it 'returns no quotes when requests filter or id arrays are empty' do
      wrap_request do
        variables = {
            'filters': [
            ],
            'ids': [
            ]
        }
        graphql url(service: service),
          format_request(
            :spSavedRecommendations,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:data][:spSavedRecommendations][:edges].length).to eq(0)
      end
    end

    it 'returns no quotes when invalid id is used in requests ids array' do
      wrap_request do
        variables = {
          'ids': [
            "invalid_id"
          ]
        }
        graphql url(service: service),
          format_request(
            :spSavedRecommendations,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:data][:spSavedRecommendations][:edges].length).to eq(0)
      end
    end

    it 'returns schema validation error when id field is ommitted' do
      wrap_request do
        variables = {
          'filters': [
            {
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
            },
            {
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :spSavedRecommendations,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
      end
    end

    it 'returns data fetching error when invalid awsAccountId is used' do
      wrap_request do
        variables = {
          'filters': [
            {
              awsAccountId: "invalid_id",
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
            },
            {
              awsAccountId: "invalid_id",
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR',
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :spSavedRecommendations,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:errorType]).to eq("BAD_REQUEST")
      end
    end

    it 'returns validation error when wrong (non-redshirt) account id is used' do
      wrap_request do
        variables = {
          'filters': [
            {
              awsAccountId: INCORRECT_CUSTOMER[:aws_billing_id],
              evaluationPeriod: 'THIRTY_DAYS',
              offeringType:  'PARTIAL_UPFRONT',
              savingsPlanTerm: 'THREE_YEAR'
            }
          ]
        }
        graphql url(service: service),
          format_request(
            :spSavedRecommendations,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("DataFetchingException")
        expect(
            json_body[:errors][0][:message]).to include(
              'AWS account is either not accessible, or does not exist within your organization, or does not have a payer account')
      end
    end
  end
end


