require 'cht-chaphql'
require 'cht_apitest_local_auth'
require 'cht_apitest_local_auth/service'
require 'shared_tests/recommendations_node_tests'
require 'shared_tests/simulation_node_tests'
require 'shared_tests/options_node_tests'

Dir[File.join(__dir__, '../lib/*', '*.rb')].each { |file| require file }
Dir[File.join(__dir__, 'helpers/*.rb')].each { |f| require f }

RSpec.configure do |c|
  c.include Chaphql
  c.include Chaphql::GraphQLHelpers
  c.include Chaphql::AnalyzerHelpers
  c.include Chaphql::StringManipulation
  c.include Chaphql::ServiceHelper
  c.include Chaphql::IntrospectConflict
  c.include ChtApitestLocalAuth
  c.include ChtApitestAuthCommon
  c.include Spaceborne
  c.include RecommendationsNodeTests
  c.include OptionsNodeTests
  c.include SimulationNodeTests
  include ChtApitestSavingsPlans
  include SavingsPlansHelpers
  c.filter_run_excluding(save_schema: true)
  c.filter_run_excluding(staging_env: true) unless ENV['TEST_ENV'] == 'staging'
  c.filter_run_excluding(not_via_proxy: true) if ENV['EKS_ENV'] == 'external'
  c.shared_context_metadata_behavior = :apply_to_host_groups
end

RSpec.shared_context 'request metadata', shared_context: :metadata do
  let(:via_public) { %w[staging canary preprod prod].include?(getenv) }
  let(:service) { ChtApitestSavingsPlans::SavingsPlansService.new }
  let(:auth_service) { LocalAuthService.new(via_public: via_public) }
  let(:tokens) { login_api(ENV['API_KEY'], service: auth_service) }
  let(:headers) do
    apitest_headers(with_auth: true, access_token: tokens[:access_token])
  end
  let(:dbg_headers) { headers.merge(use_curl: 'debug') }
  let(:get_new_tokens) do
    headers = apitest_headers(auth_service: auth_service,
                              with_auth: true,
                              refresh: tokens[:refresh_token])
    dbg_headers = headers.merge(use_curl: 'debug')
  end
end

RSpec.configure do |c|
  c.include_context 'request metadata', include_shared: true
end

def get_api_response(variables, headers, request_type)
  wrap_request do
    graphql url(service: service),
            format_request(
              request_type,
              service: service
            ),
            variables, headers
    expect_success_graphql
    # we don't run check(request_type, service: service) here
    # to account for nil values before quote completion cases
    if response.code==200 && json_body.nil?
      raise Exception, "#{request_type} api request returned 200 code and nil response"
    elsif response.code==200 && json_body.empty?
      raise Exception, "#{request_type} api request returned 200 code and empty response"
    elsif response.code==200 && json_body[:data].nil?
      raise Exception, "#{request_type} api request returned 200 code and response with no data node: #{json_body}"
    end
  end
end

def get_sp_term_as_int(term)
  if term == 'THREE_YEAR'
    3
  else
    1
  end
end

# returns object with coverageAfter value for each recommendation type
def get_coverage_per_recommendation(json_body)
  recommendation_coverage = {}
  recommendations = json_body[:data][:savingsPlanRecommendations][:recommendations][:edges]
  recommendations.each { |recommendation|
    term = recommendation[:node][:savingsPlanConfiguration][:term]
    offering_type = recommendation[:node][:savingsPlanConfiguration][:offeringType]
    recommendation_coverage[ "#{term}/#{offering_type}"] =
      recommendation[:node][:savingsPlanRecommendationSummary][:coverageAfter].round(2)
  }
  return recommendation_coverage
end

# check for presence of saved quotes, create them if they don't exist
def create_saved_recs_if_none_exist(request_body)
  # Check for the presence of the required saved quotes for the relevant tests
  variables = {'filters': request_body}
  get_api_response(variables, headers, :spSavedRecommendations)
  saved_recs = json_body[:data][:spSavedRecommendations][:edges]
  request_body.each { |request|
    request_found = false
    saved_recs.each { |saved_rec|
      if (saved_rec[:node][:evaluationPeriod] == request[:evaluationPeriod]) &&
        (saved_rec[:node][:offeringType] == request[:offeringType]) &&
        (saved_rec[:node][:awsAccountId] == request[:awsAccountId]) &&
        (saved_rec[:node][:savingsPlanTerm] == request[:savingsPlanTerm] &&
        (saved_rec[:node][:status] == 'COMPLETED'))
        request_found = true
      end
    }
    if request_found == false
      puts "did not find #{request}, creating now"
      variables = {'input':  [request]  }
      get_api_response(variables, headers, :createSpSavedRecommendations)
      # currently, the only acct we expect jobs to complete for is the redshirt main acct
      if request[:awsAccountId] == TEST_CUSTOMER[:aws_billing_id]
        quote_completion_check(json_body[:data][:createSpSavedRecommendations][:edges][0][:node][:id])
      end
    end
  }
end

# method to wait a set amount of time until a quote is completed or failed
def quote_completion_check(create_response_id)
  # the time to allow for a quote to get to COMPLETED status
  # is a combination of the wait and retry values below
  # Currently, the test allows 3 minutes for quote creation to COMPLETE
  wait_time_increment = 35 # Increase time as in preprod the time to complete is close to 10 min.
  retry_attempts = 19
  time_for_complete = wait_time_increment * (retry_attempts - 1)
  sleep(wait_time_increment)
  sp_variables = {
      'ids': [
          create_response_id
      ]
  }
  puts "getting intial status for quote #{create_response_id}"
  get_api_response(sp_variables, headers, :spSavedRecommendations)

  quote_status = json_body[:data][:spSavedRecommendations][:edges][0][:node][:status]
  if quote_status == 'FAILED'
    raise Exception, "recommendation creation job failed: #{json_body}"
  end

  attempts = 0
  wait_time = wait_time_increment
  while quote_status != 'COMPLETED' && attempts < retry_attempts
    sleep(wait_time_increment)
    wait_time += wait_time_increment
    puts "waiting #{wait_time} seconds for create quote completion (quote id: #{create_response_id}, status: #{quote_status})"
    get_api_response(sp_variables, headers, :spSavedRecommendations)
    quote_status = json_body[:data][:spSavedRecommendations][:edges][0][:node][:status]
    # fail fast if quote status goes to FAILED
    if quote_status == 'FAILED'
      raise Exception, "recommendation creation job failed: #{json_body}"
    end
    attempts += 1
  end
  if quote_status != 'COMPLETED'
    raise Exception, "Quote #{create_response_id} did not COMPLETE in #{time_for_complete} seconds, status: #{quote_status}"
  end
  # consolidated account is only redshirt account we can reliably expect to have data
  if json_body[:data][:spSavedRecommendations][:edges][0][:node][:awsAccountId] == TEST_CUSTOMER[:aws_billing_id]
    check(:spSavedRecommendations, service: service)
  end
end

TEST_CUSTOMER = {
  :aws_billing_id =>  '3367254377911',
  :owner_id => '844584796563',
  :name => 'Redshirt'
}

TEST_LINKED_ACCOUNT_2 = {
  :aws_billing_id =>  '3367254362284',
  :owner_id => '081309885503',
  :name => 'Redshirt 2'
}

TEST_LINKED_ACCOUNT_3 = {
  :aws_billing_id =>  '3367254362285',
  :owner_id => '781971168076',
  :name => 'Redshirt 3'
}

INCORRECT_CUSTOMER = {
  :aws_billing_id =>  '90',
  :owner_id => '146708650527',
  :name => 'CloudHealth'
}

COVERAGE_OVERRIDE = {
  :increase_total_coverage =>  'increase',
  :decrease_total_coverage =>  'decrease'
}

# Adding Test Customer for the Preprod

TEST_CUSTOMER_PREPROD = {
  :aws_billing_id =>  '3',
  :owner_id => '844584796563',
  :name => 'Redshirt'
}

TEST_LINKED_ACCOUNT_2_PREPROD = {
  :aws_billing_id =>  '6',
  :owner_id => '081309885503',
  :name => 'Redshirt 2'
}

TEST_LINKED_ACCOUNT_3_PREPROD  = {
  :aws_billing_id =>  '8',
  :owner_id => '781971168076',
  :name => 'Redshirt 3'
}

if ENV['DEPLOYMENT_PROVIDER'] == 'gcp'
  TEST_CUSTOMER_PREPROD[:aws_billing_id] = '4'
  TEST_CUSTOMER_PREPROD[:customer_id] = 57535
  TEST_CUSTOMER_PREPROD[:name] = 'RedshirtWS3'
  TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] = '10'
  TEST_LINKED_ACCOUNT_2_PREPROD[:name] = '081309885503'
  TEST_LINKED_ACCOUNT_3_PREPROD[:aws_billing_id] = '12'
  TEST_LINKED_ACCOUNT_3_PREPROD[:name] = '781971168076'
end

TEST_DATA_ENV = if ENV['TEST_ENV'] == 'preprod'
                  "preprod"
                elsif ENV['TEST_ENV'] == 'staging'
                  "staging"
                else
                  'default'
                end



