require 'spec_helper'
require 'json'

module SavingsPlansTest
  describe 'Positive tests: SavingsPlans updateSpSavedRecommendation' do
    include_context 'request metadata'
    desired_status = 'FAILED'

    # the following test will create a quote, then immediately update to FAILED
    # the quote is then deleted
    # Currently only testing FAILED update scenario, as RUNNING and COMPLETE
    # test cases are already subsumed by e2e test flow in create_sp_saved_recommendations_spec
    # Using desired_status variable to represent the FAILED status, as this will allow
    # easier status substitition if need to  explicitly test other status' ever arises
    it "can update status to #{desired_status}" do
      variables = {
        input: [
          {
            #awsAccountId: TEST_LINKED_ACCOUNT_2[:aws_billing_id],
            awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id],
            evaluationPeriod: 'THIRTY_DAYS',
            offeringType:  'ALL_UPFRONT',
            savingsPlanTerm: 'THREE_YEAR',
          }
        ]
      }
      # Create quote to run update on
      get_api_response(variables, headers, :createSpSavedRecommendations)
      rec_id = json_body[:data][:createSpSavedRecommendations][:edges][0][:node][:id]
      # in order to ensure that no further update calls are made for this quote
      # outside of this test (i.e. by the Scheduler while the job is running)
      # ensuring quote is completed before updating status to FAILED
      quote_completion_check(rec_id)

      initial_rec_variables = {
        'ids': [
          rec_id
        ]
      }
      # verify status is not already desired status, prior to update attempt
      get_api_response(initial_rec_variables, headers, :spSavedRecommendations)
      expect(json_body[:data][:spSavedRecommendations][:edges][0][:node][:status]).
          to_not eq(desired_status), "quote to be updated already has a #{desired_status} status : #{rec_id}"

      variables = {
        'input': {'quoteId' => rec_id, 'status' => desired_status}.to_json
      }
      get_api_response(variables, headers, :updateSpSavedRecommendation)
      check(:updateSpSavedRecommendation, service: service)
      expect(json_body[:data][:updateSpSavedRecommendation][:id]).to eq(rec_id)
      updated_recommendation_status = json_body[:data][:updateSpSavedRecommendation][:status]
      expect(updated_recommendation_status).to eq(desired_status),
        "status in updateSpSavedRecommendation response not updated for quote: #{rec_id},
         expected #{desired_status}, but got #{updated_recommendation_status}"

      # to verify data integrity, grab updated rec via the spSavedRecommendations route,
      # verify status is as expected following update
      updated_rec_variables = {
        'ids': [
          rec_id
        ]
      }
      get_api_response(updated_rec_variables, headers, :spSavedRecommendations)
      saved_recommendation_status = json_body[:data][:spSavedRecommendations][:edges][0][:node][:status]
      expect(saved_recommendation_status).to eq(desired_status),
        "status in spSavedRecommendations response not updated for quote: #{rec_id},
         expected #{desired_status}, but got #{saved_recommendation_status}"

      delete_variables = {
        recommendationId: rec_id
      }
      get_api_response(delete_variables, headers, :deleteSpSavedRecommendation)
      expect_success_graphql
      check(:deleteSpSavedRecommendation, service: service)
    end
  end

  describe 'Negative tests: SavingsPlans updateSpSavedRecommendation' do
    include_context 'request metadata'
    existing_recommendations = []

    before(:each) {
      request_body = [
        {
          awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_2_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_2[:aws_billing_id],
          evaluationPeriod: 'THIRTY_DAYS',
          offeringType:  'PARTIAL_UPFRONT',
          savingsPlanTerm: 'THREE_YEAR',
        },
        {
          awsAccountId: (TEST_DATA_ENV == 'preprod') ? TEST_LINKED_ACCOUNT_3_PREPROD[:aws_billing_id] : TEST_LINKED_ACCOUNT_3[:aws_billing_id],
          evaluationPeriod: 'THIRTY_DAYS',
          offeringType:  'ALL_UPFRONT',
          savingsPlanTerm: 'THREE_YEAR',
        }
      ]
      create_saved_recs_if_none_exist(request_body)
      variables = {'filters': request_body}
      get_api_response(variables, headers, :spSavedRecommendations)
      existing_recommendations = json_body
    }

    it 'returns expected error for no auth request' do
      rec_id = existing_recommendations[:data][:spSavedRecommendations][:edges][0][:node][:id]
      wrap_request do
        variables = {
          'input': {'quoteId' => rec_id, 'status' => 'FAILED'}.to_json
        }
        graphql url(service: service),
          format_request(
            :updateSpSavedRecommendation,
            service: service
          ),
          variables, apitest_headers
        expect_status(403)
        expect(json_body[:error]).to eq('Forbidden')
      end
    end

    it 'returns expected error when updating with invalid id' do
      rec_id = 'invalid id'
      wrap_request do
        variables = {
          'input': {'quoteId' => rec_id, 'status' => 'RUNNING'}.to_json
        }
        graphql url(service: service),
          format_request(
            :updateSpSavedRecommendation,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:data][:updateSpSavedRecommendation]).to eq(nil)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("DataFetchingException")
      end
    end

    it 'returns expected error when updating to COMPLETED without s3ManifestKey' do
      rec_id = existing_recommendations[:data][:spSavedRecommendations][:edges][0][:node][:id]
      wrap_request do
        variables = {
          'input': {'quoteId' => rec_id, 'status' => 'COMPLETED'}.to_json
        }
        graphql url(service: service),
          format_request(
            :updateSpSavedRecommendation,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:data][:updateSpSavedRecommendation]).to eq(nil)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
      end
    end

    it 'returns expected error when updating to COMPLETED with invalid s3ManifestKey' do
      rec_id = existing_recommendations[:data][:spSavedRecommendations][:edges][0][:node][:id]
      wrap_request do
        variables = {
          'input': {'quoteId' => rec_id, 'status' => 'COMPLETED', 's3ManifestKey' => 'invalid_path/recommendation-summary.json'}.to_json
        }
        graphql url(service: service),
          format_request(
            :updateSpSavedRecommendation,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:data][:updateSpSavedRecommendation]).to eq(nil)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("DataFetchingException")
      end
    end

  end
end
